package org.artifactory.federation;

import org.artifactory.repo.RepoDetails;
import org.artifactory.rest.exception.ForbiddenException;
import org.artifactory.spring.ContextCreationListener;
import org.artifactory.spring.LicenseEventListener;

import java.util.List;

/**
 * @author Alexei Vainshtein
 */
public interface FederatedRepoService extends LicenseEventListener, ContextCreationListener {

    /**
     * Fetch from each remote url the existed federated repos.
     *
     * @param urls - The remote urls to check
     * @return a list that contains a json with url and federation repos of that url.
     */
    List<FederationReposResponse> getReposFromRemote(List<String> urls, String proxyName);

    /**
     * @return a list of all federated repositories and the info per repo
     */
    List<RepoDetails> getRepos();

    /**
     * @return the token that associated with remote url and the key provided
     */
    String getToken(String remoteUrl, String key);

    /**
     * Validates the given repository
     *
     * @param key                      - repo name
     * @param clientBaseUrl            - base url of the requesting server
     * @param timeFromRequestingServer - the current time of the server that is requesting the validation.
     */
    void validate(String key, String clientBaseUrl, long timeFromRequestingServer);

    /**
     * Checks if the license provided and subscription (if AOL) are enterprise and above.
     * {@link ForbiddenException} is thrown if the license/subscription are not enterprise and above.
     */
    void validateFederationLicenseInstalled();

    /**
     * Validates remote url
     *
     * @param fullUrl - The remote url including the repository to validate
     */
    void validateRemoteServer(String fullUrl, String proxyName);

    /**
     * Migrates local repository to a federated repository
     *
     * @param repoKey - The repository name that need to be migrated.
     */
    default void migrate(String repoKey) {

    }

    default void triggerFullSync(String repoKey, String mirrorUrl) {

    }

    /**
     * Returns whether federated repo is ready to use
     */
    default boolean isActive(String repoKey) {
        return false;
    }
}
