/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.db.conversion.version.v224;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.util.function.BiPredicate;
import java.util.function.Predicate;

import static org.jfrog.storage.util.DbUtils.withMetadata;

/**
 * Predicate for replication_errors conversion into node_events_errors
 *
 * @author Mor Merhav
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class V224ConversionPredicates {
    private static final Logger log = LoggerFactory.getLogger(V224ConversionPredicates.class);

    private static final String TABLE_REPLICATION_ERRORS = "replication_errors";

    /**
     * replication_errors table conversion predicate
     */
    public static class V224ConversionPredicate implements ConversionPredicate {
        @Override
        public BiPredicate<JdbcHelper, DbType> condition() {
            return (jdbcHelper, dbType) ->
                    test(metadata -> {
                        try {
                            boolean tableExists = DbUtils.tableExists(metadata, dbType, TABLE_REPLICATION_ERRORS);
                            log.info("Does replication_errors table exists and should be converted: {}", tableExists);
                            return tableExists;
                        } catch (SQLException e) {
                            log.error("Cannot run conversion 'v224' - Failed to resolve schema metadata: ", e);
                        }
                        return false;
                    }, jdbcHelper);
        }

        private boolean test(Predicate<DatabaseMetaData> conversionPredicate, JdbcHelper jdbcHelper) {
            try {
                return withMetadata(jdbcHelper, conversionPredicate::test);
            } catch (Exception e) {
                log.error("Cannot run conversion '{}' - Failed to retrieve schema metadata: ", "v224", e);
            }
            return false;
        }
    }


}
