package org.artifactory.storage.db.conversion.version.v227;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;

import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.foreignKeyExists;

/**
 * @author Volodymyr Shulga
 */
@Slf4j
public class V227NodeEventsErrorAddConstraintConversionPredicate implements ConversionPredicate {

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                if (foreignKeyExists(jdbcHelper, dbType, "node_events_errors", "node_events_errors_event_id_fk")) {
                    log.info("Column 'event_id' already exists in table 'node_events_error' - Skipping upgrade.");
                    return false;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v227' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v227' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.info("Creating foreign key 'node_events_errors_event_id_fk' on table 'node_events_error'");
            return true;
        };
    }
}
