package org.artifactory.storage.db.conversion.version.v227;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;

import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.indexExists;

/**
 * @author Volodymyr Shulga
 */
@Slf4j
public class V227NodeEventsTmpIndexConversionPredicate implements ConversionPredicate {

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                if (indexExists(jdbcHelper, "node_events_tmp", "tmp_event_id", "node_events_tmp_idx", dbType)) {
                    log.info("Index 'node_events_tmp_idx' already exists - Skipping upgrade.");
                    return false;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v227' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v227' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.info("Creating index 'node_events_tmp_idx' on table 'node_events_tmp'");
            return true;
        };
    }
}
