package org.artifactory.storage.db.conversion.version.v227;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;

import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.tableExists;

/**
 * @author Volodymyr Shulga
 */
@Slf4j
public class V227NodeEventsTmpTableConversionPredicate implements ConversionPredicate {

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                if (tableExists(jdbcHelper, dbType, "node_events_tmp")) {
                    log.info("node_events_tmp table already exist - Skipping upgrade.");
                    return false;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v227' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v227' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.info("Creating table 'node_events_tmp'");
            return true;
        };
    }
}
