package org.artifactory.storage.db.conversion.version.v232;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;

import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.getColumnSize;

/**
 * @author Andrey Tashlitsky
 */
@Slf4j
public class V232ConversionPredicate implements ConversionPredicate {

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                // run conversion if username column size in the watches table is not 255
                if (getColumnSize(jdbcHelper, dbType, "watches", "username") < 255) {
                    log.info("Username column in watches table is less than 255 char length - Running conversion.");
                    return true;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v232' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v232' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.info("Username column in watches table is already 255 (or greater) char length.");
            return false;
        };
    }
}
