package org.artifactory.storage.db.conversion.version.v227;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.apache.commons.io.IOUtils;
import org.jfrog.storage.DbType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

/**
 * @author Volodymyr Shulga
 */
public class ConverterSplitTest {

    @Test
    void filesForAllDBsPresent() throws IOException {
        List<FileNameWithRequiredContent> requiredContents = List.of(
                new FileNameWithRequiredContent("v227_node_events_tmp_table.sql", "create table node_events_tmp"),
                new FileNameWithRequiredContent("v227_node_events_tmp_index.sql", "create index node_events_tmp_idx"),
                new FileNameWithRequiredContent("v227_node_events_add_column.sql", "alter table node_events " +
                        "add.*hints"),
                new FileNameWithRequiredContent("v227_node_events_errors_add_column.sql", "alter table " +
                        "node_events_errors add.*event_id"),
                new FileNameWithRequiredContent("v227_node_events_errors_add_constraint.sql", "alter table " +
                        "node_events_errors add constraint"),
                new FileNameWithRequiredContent("v227_node_events_errors_drop_index.sql", "drop index " +
                        "node_events_errors_op_key_idx"),
                new FileNameWithRequiredContent("v227_node_events_errors_create_index.sql",
                        "create index node_events_errors_idx")
                );
        for (DbType type : DbType.values()) {
            if (type == DbType.MARIADB) {
                continue;
            }
            for (FileNameWithRequiredContent requiredContent : requiredContents) {
                final InputStream converterStream = ConverterSplitTest.class.getResourceAsStream(
                        buildConverterPath(type, requiredContent.fileName)
                );
                Assertions.assertNotNull(converterStream, "Missing file for DbType " + type);
                final String converterContent = singleStringContent(converterStream);
                final Pattern pattern = Pattern.compile(requiredContent.requiredContentRegex);

                Assertions.assertTrue(
                        pattern.matcher(converterContent).find(),
                        String.format("Condition failed for DbType %s and fileName %s", type, requiredContent.fileName)
                );
            }
        }
    }

    @NonNull
    private String buildConverterPath(DbType type, String fileName) {
        String lowerCasedType = type.name().toLowerCase(Locale.ROOT);
        return String.join("/", "/conversion", lowerCasedType, lowerCasedType + "_" + fileName);
    }

    @NonNull
    private String singleStringContent(InputStream stream) throws IOException {
        return IOUtils.toString(stream, Charset.defaultCharset())
                .toLowerCase(Locale.ROOT)
                .replaceAll("[\r\n]+", "")
                .replaceAll("[ ]+", " ");
    }

    @AllArgsConstructor
    private static class FileNameWithRequiredContent {
        private final String fileName;
        private final String requiredContentRegex;
    }
}
