package org.artifactory.storage.db.conversion.version.v232;

import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.util.JdbcHelper;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Andrey Tashlitsky
 */
public class V232ConversionPredicateTest {

    @Test(dataProvider = "V232ConversionPredicate255ColumnSizeTestDataProvider")
    public void V232ConversionPredicate255ColumnSizeTest(int expectedColumnSize, boolean shouldConvert)
            throws SQLException {
        V232ConversionPredicate v232ConversionPredicate = new V232ConversionPredicate();
        JdbcHelper jdbcHelperMock = getJdbcHelperMock(true, expectedColumnSize);
        assertEquals(v232ConversionPredicate.condition().test(jdbcHelperMock, DbType.POSTGRESQL), shouldConvert);
    }

    @Test(expectedExceptions = DbConversionException.class,
            expectedExceptionsMessageRegExp = "Cannot run conversion 'v232' - Failed to retrieve schema metadata.*")
    public void V232ConversionPredicateSqlExceptionTest() throws SQLException {
        V232ConversionPredicate v232ConversionPredicate = new V232ConversionPredicate();
        JdbcHelper jdbcHelperMock = getJdbcHelperMock(false, 30);
        v232ConversionPredicate.condition().test(jdbcHelperMock, DbType.POSTGRESQL);
    }

    @DataProvider
    private static Object[][] V232ConversionPredicate255ColumnSizeTestDataProvider() {
        return new Object[][] {
                {64, true},
                {255, false}
        };
    }

    private JdbcHelper getJdbcHelperMock(boolean successfulColumnRetrieval, int expectedColumnSize)
            throws SQLException {
        DatabaseMetaData databaseMetaDataMock = mock(DatabaseMetaData.class);
        when(databaseMetaDataMock.getConnection()).thenReturn(mock(Connection.class));

        if (successfulColumnRetrieval) {
            ResultSet resultSetMock = mock(ResultSet.class);
            when(resultSetMock.next()).thenReturn(true);
            when(resultSetMock.getInt(any())).thenReturn(expectedColumnSize);
            when(databaseMetaDataMock.getColumns(any(), any(), any(), any())).thenReturn(resultSetMock);
        } else {
            when(databaseMetaDataMock.getColumns(any(), any(), any(), any())).thenThrow(SQLException.class);
        }

        Connection connectionMock = mock(Connection.class);
        when(connectionMock.getMetaData()).thenReturn(databaseMetaDataMock);
        JFrogDataSource dataSourceMock = mock(JFrogDataSource.class);
        when(dataSourceMock.getConnection()).thenReturn(connectionMock);
        JdbcHelper jdbcHelperMock = mock(JdbcHelper.class);
        when(jdbcHelperMock.getDataSource()).thenReturn(dataSourceMock);
        return jdbcHelperMock;
    }
}
