package org.artifactory.storage.db.conversion.version.v233;

import org.artifactory.storage.db.util.JdbcHelper;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;
import org.testng.annotations.Test;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

public class V233ConversionPredicateTest {

    @Test
    public void whenColumnExistsThenNoConversionNeeded() throws SQLException {
        V233ConversionPredicate predicate = new V233ConversionPredicate();
        JdbcHelper helper = getJdbcHelperMock(true);
        assertFalse(predicate.condition().test(helper, DbType.DERBY));
    }

    @Test
    public void whenColumnNotExistsThenConversionNeeded() throws SQLException {
        V233ConversionPredicate predicate = new V233ConversionPredicate();
        JdbcHelper helper = getJdbcHelperMock(false);
        assertTrue(predicate.condition().test(helper, DbType.DERBY));
    }

    private JdbcHelper getJdbcHelperMock(boolean exists) throws SQLException {
        DatabaseMetaData databaseMetaDataMock = mock(DatabaseMetaData.class);
        when(databaseMetaDataMock.getConnection()).thenReturn(mock(Connection.class));

        ResultSet resultSetMock = mock(ResultSet.class);
        when(resultSetMock.next()).thenReturn(exists);
        when(databaseMetaDataMock.getColumns(any(), any(), any(), any())).thenReturn(resultSetMock);

        Connection connectionMock = mock(Connection.class);
        when(connectionMock.getMetaData()).thenReturn(databaseMetaDataMock);

        JFrogDataSource dataSourceMock = mock(JFrogDataSource.class);
        when(dataSourceMock.getConnection()).thenReturn(connectionMock);

        JdbcHelper jdbcHelperMock = mock(JdbcHelper.class);
        when(jdbcHelperMock.getDataSource()).thenReturn(dataSourceMock);
        return jdbcHelperMock;
    }
}
